﻿using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Data;

namespace HeuristicLab.Algorithms.NSGA3
{
    internal static class Utility
    {
        /// <summary>
        /// Returns the number of possible combinations of size <paramref name="r" /> from a set of
        /// size <paramref name="n" />.
        /// </summary>
        /// <param name="n"></param>
        /// <param name="r"></param>
        /// <returns></returns>
        /// <exception cref="ArgumentException">
        /// Thrown, when <paramref name="r" /> &gt; <paramref name="n" /> or when <paramref name="r"
        /// /> &lt; 1.
        /// </exception>
        internal static int NCR(int n, int r)
        {
            if (n < r) throw new ArgumentException($"Constraint was not met: n >= r (n = {n}, r = {r})");
            if (r < 1) throw new ArgumentException($"Constraint was not met: r >= 1 (r = {r})");
            if (n == r) return n;
            return Factorial(n) / (Factorial(r) * Factorial(n - r));
        }

        internal static int Factorial(int n)
        {
            if (n <= 0) throw new ArgumentException($"Constraint for n was not met: 0 < n <= 30 (n = {n})");
            int product = 1;
            for (int i = 2; i <= n; i++)
                product *= i;

            return product;
        }

        internal static DoubleMatrix ConvertToDoubleMatrix(List<ReferencePoint> referencePoints)
        {
            return new DoubleMatrix(ToArray(referencePoints));
        }

        internal static double[,] ToArray(List<ReferencePoint> referencePoints)
        {
            if (referencePoints == null || !referencePoints.Any()) throw new ArgumentException($"{nameof(referencePoints)} is null or empty");
            int nDim = referencePoints.First().NumberOfDimensions;
            int pointCount = referencePoints.Count;
            double[,] array = new double[nDim, pointCount];

            for (int p = 0; p < pointCount; p++)
                for (int d = 0; d < nDim; d++)
                    array[d, p] = referencePoints[p].Values[d];

            return array;
        }

        internal static DoubleMatrix ToMatrix(this IEnumerable<IReadOnlyList<double>> data)
        {
            var d2 = data.ToArray();
            var mat = new DoubleMatrix(d2.Length, d2[0].Count);
            for (var i = 0; i < mat.Rows; i++)
                for (var j = 0; j < mat.Columns; j++)
                    mat[i, j] = d2[i][j];
            return mat;
        }
    }
}